library ieee;
use ieee.std_logic_1164.all;

entity dp is
generic(
	N : integer := 4
);
port(   
	rst, clk	: in  std_logic;
	first, second, count	: in  std_logic_vector(N-1 downto 0); 
	first_sel, second_sel : in  std_logic;
	first_load, second_load, counter_load : in  std_logic;
	comp_out	: out std_logic;  
	output	: out std_logic_vector(N-1 downto 0)
);
end dp;

architecture dp_arch of dp is

component mux is
generic(
	N : integer :=4
);
port(
	sel	: in std_logic;
	x, y	: in  std_logic_vector(N-1 downto 0);
	output	: out std_logic_vector(N-1 downto 0)
);
end component;

component reg is
generic(
	N : integer := 4
);
port(
	rst, clk, load          :in  std_logic;
	input                   :in  std_logic_vector(N-1 downto 0);
	output                  :out std_logic_vector(N-1 downto 0)
);
end component;

component add is
generic(
	N : integer := 4
);
port(
	x, y	: in std_logic_vector(N-1 downto 0);
	output	: out std_logic_vector(N-1 downto 0)
);
end component;

component comp is
generic(
	N : integer := 4
);
port(
	x            :in  std_logic_vector(N-1 downto 0);
	output       :out std_logic
);
end component;

component counter is
generic(
	N : integer := 4
);
port(
	clk, rst  :in  std_logic;
	input	:in  std_logic_vector(N-1 downto 0);
	load	:in  std_logic;
	output	:out std_logic_vector(N-1 downto 0)
);
end component;

	signal first_reg, second_reg, add_out, counter_r, first_r, second_r : std_logic_vector(N-1 downto 0);
begin
	b1:	mux generic map(N => N) port map(sel => first_sel, x => first, y => second_r, output => first_reg);
	b2:	mux generic map(N => N) port map(sel => second_sel, x => second, y => add_out, output => second_reg);
	b3: reg generic map(N => N) port map(rst => rst, clk => clk, load => first_load , input => first_reg, output => first_r);
	b4: reg generic map(N => N) port map(rst => rst, clk => clk, load => second_load , input => second_reg, output => second_r);
	b5: add generic map(N => N) port map(x => first_r , y => second_r, output => add_out);
	b6: counter generic map(N => N) port map(rst => rst, clk => clk, input => count, load => counter_load , output => counter_r);
	b7:	comp generic map(N => N) port map(x => counter_r, output => comp_out);
	output <= second_r;
end dp_arch;