library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity fsm is
port(
	rst, clk	: in std_logic;
	comp, start	: in std_logic;
	first_sel, second_sel : out  std_logic;
	first_load, second_load, counter_load : out  std_logic;
	ready	: out std_logic
);
end fsm;

architecture fsm_arch of fsm is
	type state_type is (init, s1, s2, last);
	signal state_reg, state_next : state_type;
begin
	process(rst, clk)
	begin 
		if rst = '1' then
			state_reg <= init;
		elsif rising_edge(clk) then 
			state_reg <= state_next;
		end if; 
	end process;

	process(state_reg, comp, start)
	begin 
		ready <= '0';
		
		first_sel <= '0';
		second_sel <= '0';
		first_load <= '0';
		second_load <= '0';
		counter_load <= '0';

		case state_reg is
			when init =>
				if start = '0' then
					state_next <= init;
				else
					state_next <= s1;
				end if;
			when s1 =>
				first_load <= '1';
				second_load <= '1';
				counter_load <= '1';
				state_next <= s2;
			when s2 =>
				first_sel <= '1';
				second_sel <= '1';
				first_load <= '1';
				second_load <= '1';

				if comp = '1' then
					state_next <= s2;
				else
					state_next <= last;
				end if;
			when last =>
				ready <= '1';
				state_next <= init;
			when others => 
				state_next <= init;
      end case;
  end process;
end fsm_arch;